#import "KGOBookmarksViewController.h"
#import "KGOSearchDisplayController.h"

@implementation KGOBookmarksViewController

@synthesize bookmarkedItems, searchResultsDelegate;

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc
{
    [_mutableItems release];
    self.searchResultsDelegate = nil;
    self.bookmarkedItems = nil;
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.navigationItem.leftBarButtonItem = self.editButtonItem;
    self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone
                                                                                            target:self
                                                                                            action:@selector(dismissModalViewControllerAnimated:)] autorelease];
    
    _mutableItems = [self.bookmarkedItems mutableCopy];
    self.navigationItem.leftBarButtonItem.enabled = _mutableItems.count > 0;
    
    self.title = NSLocalizedString(@"Bookmarks", @"title of generic bookmark view controller");
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation
{
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
        return YES;
    }
    return toInterfaceOrientation == UIInterfaceOrientationPortrait;
}

#pragma mark KGOSearchResultsHolder

- (NSArray *)results
{
    return self.bookmarkedItems;
}

- (void)receivedSearchResults:(NSArray *)results forSource:(NSString *)source
{
    self.bookmarkedItems = results;
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return _mutableItems.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
    id<KGOSearchResult> aResult = [_mutableItems objectAtIndex:indexPath.row];
    cell.textLabel.text = aResult.title;
    if ([aResult respondsToSelector:@selector(subtitle)]) {
        cell.detailTextLabel.text = [aResult subtitle];
    }
    
    return cell;
}


- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        id<KGOSearchResult> aResult = [_mutableItems objectAtIndex:indexPath.row];
        [aResult removeBookmark];
        [_mutableItems removeObject:aResult];
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
        
        if (!_mutableItems.count) {
            self.navigationItem.leftBarButtonItem.enabled = NO;
        }
    }
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    id<KGOSearchResult> aResult = [_mutableItems objectAtIndex:indexPath.row];
    [self.searchResultsDelegate resultsHolder:self didSelectResult:aResult];
    [self dismissModalViewControllerAnimated:YES];
}

@end
